SET ECHO ON TIMING ON TIME ON

-- ***** CONNECT STAGE

CONNECT STAGE

-- ***** BEGIN Script:	redef_TEST_REDEF.sql

WHENEVER SQLERROR EXIT SQL.SQLCODE

SPOOL redef_TEST_REDEF.log

-- ***** Run CAN_REDEF_TABLE to see if this table can be redefined

BEGIN
	 DBMS_REDEFINITION.CAN_REDEF_TABLE (	
   		'STAGE',		
   		'TEST_REDEF',			
   		DBMS_REDEFINITION.CONS_USE_PK	
	);
END;
/

-- ***** Create Interim Table

START cr_TEST_REDEF_COPY.sql

-- ***** Change Parallel Degree for Interim Table to be used during start_redef_table

ALTER TABLE TEST_REDEF_COPY PARALLEL (DEGREE 4);

ALTER SESSION ENABLE PARALLEL DML;

-- ***** Run START_REDEF_TABLE

BEGIN
	 DBMS_REDEFINITION.START_REDEF_TABLE (	
   		'STAGE',		
   		'TEST_REDEF',		
   		'TEST_REDEF_COPY',	
   		NULL,				
   		DBMS_REDEFINITION.CONS_USE_PK	
	);
END;
/

-- ***** Change Parallel Degree for Interim Table back to 1

ALTER TABLE TEST_REDEF_COPY PARALLEL (DEGREE 1);

-- ***** Create PK Index on Interim Table

START cr_TEST_REDEF_COPY_pk_ix.sql

-- ***** Run SYNC_INTERIM_TABLE (for any changes that may have happened during build PK Index)

BEGIN
	 DBMS_REDEFINITION.SYNC_INTERIM_TABLE (	
   		'STAGE',		
   		'TEST_REDEF',		
   		'TEST_REDEF_COPY'		
	);
END;
/

-- ***** Create All Other Indexes on Interim Table

START cr_TEST_REDEF_COPY_ix.sql

-- ***** Run SYNC_INTERIM_TABLE (for any changes that may have happened during Other Index builds)

BEGIN
	 DBMS_REDEFINITION.SYNC_INTERIM_TABLE (	
   		'STAGE',		
   		'TEST_REDEF',		
   		'TEST_REDEF_COPY'		
	);
END;
/

-- ***** Create PK on Interim Table

START cr_TEST_REDEF_COPY_pk.sql

-- ***** Run SYNC_INTERIM_TABLE (for any changes that may have happened during PK creations)

BEGIN
	 DBMS_REDEFINITION.SYNC_INTERIM_TABLE (	
   		'STAGE',		
   		'TEST_REDEF',		
   		'TEST_REDEF_COPY'		
	);
END;
/

-- ***** Display the tablespace_names for the 2 Tables BEFORE FINISH_REDEF_TABLE

SET LINES 132

COLUMN SEGMENT_NAME FORMAT A32
COLUMN TABLESPACE_NAME FORMAT A32
COLUMN SIZE_MB FORMAT 9,999,999

SELECT SEGMENT_NAME, TABLESPACE_NAME, ROUND(SUM(BYTES/1024/1024),2) SIZE_MB
FROM	USER_SEGMENTS
WHERE SEGMENT_NAME IN ( 'TEST_REDEF', 'TEST_REDEF_COPY' )
GROUP BY SEGMENT_NAME, TABLESPACE_NAME
/

-- ***** Run FINISH_REDEF_TABLE (to rename 'INTERIM to ACTUAL' and vice-versa)

BEGIN
	 DBMS_REDEFINITION.FINISH_REDEF_TABLE (	
   		'STAGE',		
   		'TEST_REDEF',		
   		'TEST_REDEF_COPY'		
	);
END;
/

-- ***** Display the tablespace_names for the 2 Tables AFTER FINISH_REDEF_TABLE

SELECT SEGMENT_NAME, TABLESPACE_NAME, ROUND(SUM(BYTES/1024/1024),2) SIZE_MB
FROM	USER_SEGMENTS
WHERE SEGMENT_NAME IN ( 'TEST_REDEF', 'TEST_REDEF_COPY' )
GROUP BY SEGMENT_NAME, TABLESPACE_NAME
/

-- ***** PERFORM DROP ORIGINAL TABLE, RENAME CONSTRAINTS ETC.

START rename_TEST_REDEF_cons.sql

-- ***** Display the tablespace_names AFTER DROP Table

SELECT SEGMENT_NAME, TABLESPACE_NAME, ROUND(SUM(BYTES/1024/1024),2) MB
FROM   USER_SEGMENTS A1
WHERE SEGMENT_NAME IN ( 'TEST_REDEF', 'TEST_REDEF_COPY' )
   OR SEGMENT_NAME IN (
  	 		SELECT INDEX_NAME
			FROM USER_INDEXES B1
			WHERE B1.TABLE_NAME IN ('TEST_REDEF', 'TEST_REDEF_COPY') )
GROUP BY SEGMENT_NAME, TABLESPACE_NAME
ORDER BY 1, 2
/

SPOOL OFF

-- ***** END Script: redef_TEST_REDEF.sql

